@@ -1,4 +1,4 @@
-GDALinfo <- function(fname, silent=FALSE, returnRAT=FALSE) {
+GDALinfo <- function(fname, silent=FALSE, returnRAT=FALSE, returnCategoryNames=FALSE) {
 	if (nchar(fname) == 0) stop("empty file name")
 	x <- GDAL.open(fname, silent=silent)
 	d <- dim(x)[1:2]
@@ -26,6 +26,8 @@ GDALinfo <- function(fname, silent=FALSE, returnRAT=FALSE) {
             hasNoDataValues <- logical(nbands)
             NoDataValues <- numeric(nbands)
             if (returnRAT) RATlist <- vector(mode="list", length=nbands)
+            if (returnCategoryNames)
+                CATlist <- vector(mode="list", length=nbands)
             for (i in seq(along = band)) {
 
                 raster <- getRasterBand(x, band[i])
@@ -48,6 +50,11 @@ GDALinfo <- function(fname, silent=FALSE, returnRAT=FALSE) {
                     RATi <- .Call("RGDAL_GetRAT", raster, PACKAGE="rgdal")
                     if (!is.null(RATi)) RATlist[[i]] <- RATi
                 }
+                if (returnCategoryNames) {
+                    CATi <- .Call("RGDAL_GetCategoryNames", raster,
+                        PACKAGE="rgdal")
+                    if (!is.null(CATi)) CATlist[[i]] <- CATi
+                }
                 NDV <- .Call("RGDAL_GetBandNoDataValue", raster,
                     PACKAGE="rgdal")
                 if (is.null(NDV)) {
@@ -85,6 +92,7 @@ GDALinfo <- function(fname, silent=FALSE, returnRAT=FALSE) {
         attr(res, "mdata") <- mdata
         attr(res, "subdsmdata") <- subdsmdata
         if (returnRAT) attr(res, "RATlist") <- RATlist
+        if (returnCategoryNames) attr(res, "CATlist") <- CATlist
 	class(res) <- "GDALobj"
 	res
 }
@@ -133,6 +141,12 @@ print.GDALobj <- function(x, ...) {
                     collapse=", ")), "\n")
             }
         }
+        if (!is.null(attr(x, "CATlist"))) {
+            CATs <- attr(x, "CATlist")
+            nCAT <- length(CATs)
+            cat("Category names:\n")
+            print(CATs)            
+        }
 	invisible(x)
 }
 
@@ -157,7 +171,7 @@ asGDALROD_SGDF <- function(from) {
 		abs(cellsize[2]))
 	cellcentre.offset <- c(x=co.x, y=co.y)
 	grid = GridTopology(cellcentre.offset, cellsize, rev(output.dim))
-	if (length(d) == 2)
+	if (length(d) == 2L)
 		df = list(band1 = as.vector(data))
 	else {
 		df <- vector(mode="list", length=d[3])
@@ -208,7 +222,7 @@ asSGDF_GROD <- function(x, offset, region.dim, output.dim, p4s=NULL, ..., half.c
 		abs(cellsize[2]))
 	cellcentre.offset <- c(x=co.x, y=co.y)
 	grid = GridTopology(cellcentre.offset, cellsize, rev(output.dim))
-	if (length(d) == 2)
+	if (length(d) == 2L)
 		df = list(band1 = as.vector(data))
 	else {
 		df <- vector(mode="list", length=d[3])
@@ -233,7 +247,7 @@ readGDAL = function(fname, offset, region.dim, output.dim, band, p4s=NULL, ...,
 	odim_flag <- NULL
         if (missing(band)) band <- NULL
         else {
-                if (length(band) > 1) d[3] <- length(band)
+                if (length(band) > 1L) d[3] <- length(band)
                 else d <- d[1:2]
         }
 # bug report Mike Sumner 070522
@@ -291,7 +305,7 @@ readGDAL = function(fname, offset, region.dim, output.dim, band, p4s=NULL, ...,
 		data = getRasterData(x, band=band, offset=offset, 
 			region.dim=region.dim, output.dim=output.dim, ...)
 		GDAL.close(x)
-		if (length(d) == 2)
+		if (length(d) == 2L)
 			df = list(band1 = as.vector(data))
 		else {
 			df <- vector(mode="list", length=d[3])
